@@ -1,9 +1,10 @@
 # --
-# Copyright (C) 2001-2020 OTRS AG, https://otrs.com/
+# Kernel/Modules/AdminSystemMaintenance.pm - to control all system maintenance actions
+# Copyright (C) 2001-2015 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::Modules::AdminSystemMaintenance;
@@ -11,10 +12,10 @@ package Kernel::Modules::AdminSystemMaintenance;
 use strict;
 use warnings;
 
-use Kernel::System::VariableCheck qw(:all);
-use Kernel::Language qw(Translatable);
+use Kernel::System::Valid;
+use Kernel::System::SystemMaintenance;
 
-our $ObjectManagerDisabled = 1;
+use Kernel::System::VariableCheck qw(:all);
 
 sub new {
     my ( $Type, %Param ) = @_;
@@ -23,19 +24,25 @@ sub new {
     my $Self = {%Param};
     bless( $Self, $Type );
 
+    # check needed objects
+    for my $Needed (qw(ParamObject DBObject LayoutObject LogObject ConfigObject TimeObject)) {
+        if ( !$Self->{$Needed} ) {
+            $Self->{LayoutObject}->FatalError( Message => "Got no $Needed!" );
+        }
+    }
+
+    # create additional objects
+    $Self->{ValidObject}             = Kernel::System::Valid->new(%Param);
+    $Self->{SystemMaintenanceObject} = Kernel::System::SystemMaintenance->new( %{$Self} );
+
     return $Self;
 }
 
 sub Run {
     my ( $Self, %Param ) = @_;
 
-    my $LayoutObject            = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
-    my $ParamObject             = $Kernel::OM->Get('Kernel::System::Web::Request');
-    my $SessionObject           = $Kernel::OM->Get('Kernel::System::AuthSession');
-    my $SystemMaintenanceObject = $Kernel::OM->Get('Kernel::System::SystemMaintenance');
-
-    my $SystemMaintenanceID = $ParamObject->GetParam( Param => 'SystemMaintenanceID' ) || '';
-    my $WantSessionID       = $ParamObject->GetParam( Param => 'WantSessionID' )       || '';
+    my $SystemMaintenanceID = $Self->{ParamObject}->GetParam( Param => 'SystemMaintenanceID' ) || '';
+    my $WantSessionID       = $Self->{ParamObject}->GetParam( Param => 'WantSessionID' )       || '';
 
     my $SessionVisibility = 'Collapsed';
 
@@ -45,10 +52,10 @@ sub Run {
     if ( $Self->{Subaction} eq 'Kill' ) {
 
         # challenge token check for write action
-        $LayoutObject->ChallengeTokenCheck();
+        $Self->{LayoutObject}->ChallengeTokenCheck();
 
-        $SessionObject->RemoveSessionID( SessionID => $WantSessionID );
-        return $LayoutObject->Redirect(
+        $Self->{SessionObject}->RemoveSessionID( SessionID => $WantSessionID );
+        return $Self->{LayoutObject}->Redirect(
             OP =>
                 "Action=AdminSystemMaintenance;Subaction=SystemMaintenanceEdit;SystemMaintenanceID=$SystemMaintenanceID;Kill=1"
         );
@@ -60,16 +67,16 @@ sub Run {
     elsif ( $Self->{Subaction} eq 'KillAll' ) {
 
         # challenge token check for write action
-        $LayoutObject->ChallengeTokenCheck();
+        $Self->{LayoutObject}->ChallengeTokenCheck();
 
-        my @Sessions = $SessionObject->GetAllSessionIDs();
+        my @Sessions = $Self->{SessionObject}->GetAllSessionIDs();
         SESSIONS:
         for my $Session (@Sessions) {
             next SESSIONS if $Session eq $WantSessionID;
-            $SessionObject->RemoveSessionID( SessionID => $Session );
+            $Self->{SessionObject}->RemoveSessionID( SessionID => $Session );
         }
 
-        return $LayoutObject->Redirect(
+        return $Self->{LayoutObject}->Redirect(
             OP =>
                 "Action=AdminSystemMaintenance;Subaction=SystemMaintenanceEdit;SystemMaintenanceID=$SystemMaintenanceID;KillAll=1"
         );
@@ -92,7 +99,7 @@ sub Run {
     elsif ( $Self->{Subaction} eq 'SystemMaintenanceNewAction' ) {
 
         # challenge token check for write action
-        $LayoutObject->ChallengeTokenCheck();
+        $Self->{LayoutObject}->ChallengeTokenCheck();
 
         # check required parameters
         my %Error;
@@ -119,7 +126,7 @@ sub Run {
             # add notification
             push @NotifyData, {
                 Priority => 'Error',
-                Info     => Translatable('Start date shouldn\'t be defined after Stop date!'),
+                Info     => "Start date shouldn't be defined after Stop date!",
             };
         }
 
@@ -130,17 +137,6 @@ sub Run {
 
         }
 
-        # Trigger server error if 'Login message' or 'Notify message' is longer then 250 characters.
-        #   See bug#13366 (https://bugs.otrs.org/show_bug.cgi?id=13366)
-        if ( $SystemMaintenanceData->{LoginMessage} && length $SystemMaintenanceData->{LoginMessage} > 250 ) {
-
-            $Error{LoginMessageServerError} = 'ServerError';
-        }
-        if ( $SystemMaintenanceData->{NotifyMessage} && length $SystemMaintenanceData->{NotifyMessage} > 250 ) {
-
-            $Error{NotifyMessageServerError} = 'ServerError';
-        }
-
         if ( !$SystemMaintenanceData->{ValidID} ) {
 
             # add server error class
@@ -159,7 +155,7 @@ sub Run {
             );
         }
 
-        my $SystemMaintenanceID = $SystemMaintenanceObject->SystemMaintenanceAdd(
+        my $SystemMaintenanceID = $Self->{SystemMaintenanceObject}->SystemMaintenanceAdd(
             StartDate        => $SystemMaintenanceData->{StartDate},
             StopDate         => $SystemMaintenanceData->{StopDate},
             Comment          => $SystemMaintenanceData->{Comment},
@@ -172,25 +168,16 @@ sub Run {
 
         # show error if can't create
         if ( !$SystemMaintenanceID ) {
-            return $LayoutObject->ErrorScreen(
-                Message => Translatable('There was an error creating the System Maintenance'),
+            return $Self->{LayoutObject}->ErrorScreen(
+                Message => "There was an error creating the SystemMaintenance",
             );
         }
 
         # redirect to edit screen
-        if (
-            defined $ParamObject->GetParam( Param => 'ContinueAfterSave' )
-            && ( $ParamObject->GetParam( Param => 'ContinueAfterSave' ) eq '1' )
-            )
-        {
-            return $LayoutObject->Redirect(
-                OP =>
-                    "Action=$Self->{Action};Subaction=SystemMaintenanceEdit;SystemMaintenanceID=$SystemMaintenanceID;Notification=Add"
-            );
-        }
-        else {
-            return $LayoutObject->Redirect( OP => "Action=$Self->{Action};Notification=Add" );
-        }
+        return $Self->{LayoutObject}->Redirect(
+            OP =>
+                "Action=$Self->{Action};Subaction=SystemMaintenanceEdit;SystemMaintenanceID=$SystemMaintenanceID;Saved=1"
+        );
     }
 
     # ------------------------------------------------------------ #
@@ -203,13 +190,13 @@ sub Run {
 
         # check for SystemMaintenanceID
         if ( !$SystemMaintenanceID ) {
-            return $LayoutObject->ErrorScreen(
-                Message => Translatable('Need SystemMaintenanceID!'),
+            return $Self->{LayoutObject}->ErrorScreen(
+                Message => "Need SystemMaintenanceID!",
             );
         }
 
         # get system maintenance data
-        my $SystemMaintenanceData = $SystemMaintenanceObject->SystemMaintenanceGet(
+        my $SystemMaintenanceData = $Self->{SystemMaintenanceObject}->SystemMaintenanceGet(
             ID     => $SystemMaintenanceID,
             UserID => $Self->{UserID},
         );
@@ -217,63 +204,46 @@ sub Run {
         # include time stamps on the correct key
         for my $Key (qw(StartDate StopDate)) {
 
-            # try to convert to TimeStamp
-            my $DateTimeObject = $Kernel::OM->Create(
-                'Kernel::System::DateTime',
-                ObjectParams => {
-                    Epoch => $SystemMaintenanceData->{$Key},
-                }
+            # try to convert SystemTime to TimeStamp
+            $SystemMaintenanceData->{ $Key . 'TimeStamp' } = $Self->{TimeObject}->SystemTime2TimeStamp(
+                SystemTime => $SystemMaintenanceData->{$Key},
             );
-            $SystemMaintenanceData->{ $Key . 'TimeStamp' } =
-                $DateTimeObject ? $DateTimeObject->ToString() : undef;
         }
 
         # check for valid system maintenance data
         if ( !IsHashRefWithData($SystemMaintenanceData) ) {
-            return $LayoutObject->ErrorScreen(
-                Message => $LayoutObject->{LanguageObject}->Translate(
-                    'Could not get data for SystemMaintenanceID %s',
-                    $SystemMaintenanceID
-                ),
+            return $Self->{LayoutObject}->ErrorScreen(
+                Message => "Could not get data for SystemMaintenanceID $SystemMaintenanceID",
             );
         }
 
-        if ( $ParamObject->GetParam( Param => 'Notification' ) eq 'Add' ) {
+        if ( $Self->{ParamObject}->GetParam( Param => 'Saved' ) ) {
 
             # add notification
             push @NotifyData, {
                 Priority => 'Notice',
-                Info     => Translatable('System Maintenance was added successfully!'),
+                Info     => "System Maintenance was saved successfully!",
             };
         }
 
-        if ( $ParamObject->GetParam( Param => 'Notification' ) eq 'Update' ) {
+        if ( $Self->{ParamObject}->GetParam( Param => 'Kill' ) ) {
 
             # add notification
             push @NotifyData, {
                 Priority => 'Notice',
-                Info     => Translatable('System Maintenance was updated successfully!'),
-            };
-        }
-
-        if ( $ParamObject->GetParam( Param => 'Kill' ) ) {
-
-            # add notification
-            push @NotifyData, {
-                Priority => 'Notice',
-                Info     => Translatable('Session has been killed!'),
+                Info     => 'Session has been killed!',
             };
 
             # set class for expanding sessions widget
             $SessionVisibility = 'Expanded';
         }
 
-        if ( $ParamObject->GetParam( Param => 'KillAll' ) ) {
+        if ( $Self->{ParamObject}->GetParam( Param => 'KillAll' ) ) {
 
             # add notification
             push @NotifyData, {
                 Priority => 'Notice',
-                Info     => Translatable('All sessions have been killed, except for your own.'),
+                Info     => 'All sessions has been killed, exept the current one!',
             };
 
             # set class for expanding sessions widget
@@ -297,7 +267,7 @@ sub Run {
     elsif ( $Self->{Subaction} eq 'SystemMaintenanceEditAction' ) {
 
         # challenge token check for write action
-        $LayoutObject->ChallengeTokenCheck();
+        $Self->{LayoutObject}->ChallengeTokenCheck();
 
         # check required parameters
         my %Error;
@@ -322,7 +292,7 @@ sub Run {
             # add notification
             push @NotifyData, {
                 Priority => 'Error',
-                Info     => Translatable('Start date shouldn\'t be defined after Stop date!'),
+                Info     => "Start date shouldn't be defined after Stop date!",
             };
         }
 
@@ -333,17 +303,6 @@ sub Run {
 
         }
 
-        # Trigger server error if 'Login message' or 'Notify message' is longer then 250 characters.
-        #   See bug#13366 (https://bugs.otrs.org/show_bug.cgi?id=13366)
-        if ( $SystemMaintenanceData->{LoginMessage} && length $SystemMaintenanceData->{LoginMessage} > 250 ) {
-
-            $Error{LoginMessageServerError} = 'ServerError';
-        }
-        if ( $SystemMaintenanceData->{NotifyMessage} && length $SystemMaintenanceData->{NotifyMessage} > 250 ) {
-
-            $Error{NotifyMessageServerError} = 'ServerError';
-        }
-
         if ( !$SystemMaintenanceData->{ValidID} ) {
 
             # add server error class
@@ -364,7 +323,7 @@ sub Run {
         }
 
         # otherwise update configuration and return to edit screen
-        my $UpdateResult = $SystemMaintenanceObject->SystemMaintenanceUpdate(
+        my $UpdateResult = $Self->{SystemMaintenanceObject}->SystemMaintenanceUpdate(
             ID               => $SystemMaintenanceID,
             StartDate        => $SystemMaintenanceData->{StartDate},
             StopDate         => $SystemMaintenanceData->{StopDate},
@@ -378,26 +337,16 @@ sub Run {
 
         # show error if can't create
         if ( !$UpdateResult ) {
-            return $LayoutObject->ErrorScreen(
-                Message => Translatable('There was an error updating the System Maintenance'),
+            return $Self->{LayoutObject}->ErrorScreen(
+                Message => "There was an error updating the SystemMaintenance",
             );
         }
 
         # redirect to edit screen
-        if (
-            defined $ParamObject->GetParam( Param => 'ContinueAfterSave' )
-            && ( $ParamObject->GetParam( Param => 'ContinueAfterSave' ) eq '1' )
-            )
-        {
-            return $LayoutObject->Redirect(
-                OP =>
-                    "Action=$Self->{Action};Subaction=SystemMaintenanceEdit;SystemMaintenanceID=$SystemMaintenanceID;Notification=Update"
-            );
-        }
-        else {
-            return $LayoutObject->Redirect( OP => "Action=$Self->{Action};Notification=Update" );
-        }
-
+        return $Self->{LayoutObject}->Redirect(
+            OP =>
+                "Action=$Self->{Action};Subaction=SystemMaintenanceEdit;SystemMaintenanceID=$SystemMaintenanceID;Saved=1"
+        );
     }
 
     # ------------------------------------------------------------ #
@@ -406,28 +355,26 @@ sub Run {
     elsif ( $Self->{Subaction} eq 'Delete' ) {
 
         # challenge token check for write action
-        $LayoutObject->ChallengeTokenCheck();
+        $Self->{LayoutObject}->ChallengeTokenCheck();
 
         if ( !$SystemMaintenanceID ) {
-            $Kernel::OM->Get('Kernel::System::Log')->Log(
+            $Self->{LogObject}->Log(
                 Message  => "No System Maintenance ID $SystemMaintenanceID",
                 Priority => 'error',
             );
         }
 
-        my $Delete = $SystemMaintenanceObject->SystemMaintenanceDelete(
+        my $Delete = $Self->{SystemMaintenanceObject}->SystemMaintenanceDelete(
             ID     => $SystemMaintenanceID,
             UserID => $Self->{UserID},
         );
         if ( !$Delete ) {
-            return $LayoutObject->ErrorScreen(
-                Message => $LayoutObject->{LanguageObject}->Translate(
-                    'Was not possible to delete the SystemMaintenance entry: %s!',
-                    $SystemMaintenanceID
-                ),
+            return $Self->{LayoutObject}->ErrorScreen(
+                Message =>
+                    "Was not possible to delete the SystemMaintenance entry : $SystemMaintenanceID!",
             );
         }
-        return $LayoutObject->Redirect( OP => 'Action=AdminSystemMaintenance' );
+        return $Self->{LayoutObject}->Redirect( OP => 'Action=AdminSystemMaintenance' );
 
     }
 
@@ -436,14 +383,14 @@ sub Run {
     # ------------------------------------------------------------ #
     else {
 
-        my $SystemMaintenanceList = $SystemMaintenanceObject->SystemMaintenanceListGet(
+        my $SystemMaintenanceList = $Self->{SystemMaintenanceObject}->SystemMaintenanceListGet(
             UserID => $Self->{UserID},
         );
 
         if ( !scalar @{$SystemMaintenanceList} ) {
 
             # no data found block
-            $LayoutObject->Block(
+            $Self->{LayoutObject}->Block(
                 Name => 'NoDataRow',
             );
         }
@@ -452,25 +399,20 @@ sub Run {
             for my $SystemMaintenance ( @{$SystemMaintenanceList} ) {
 
                 # set the valid state
-                $SystemMaintenance->{ValidID} = $Kernel::OM->Get('Kernel::System::Valid')
-                    ->ValidLookup( ValidID => $SystemMaintenance->{ValidID} );
+                $SystemMaintenance->{ValidID}
+                    = $Self->{ValidObject}->ValidLookup( ValidID => $SystemMaintenance->{ValidID} );
 
                 # include time stamps on the correct key
                 for my $Key (qw(StartDate StopDate)) {
 
-                    my $DateTimeObject = $Kernel::OM->Create(
-                        'Kernel::System::DateTime',
-                        ObjectParams => {
-                            Epoch => $SystemMaintenance->{$Key},
-                        },
+                    # try to convert SystemTime to TimeStamp
+                    $SystemMaintenance->{ $Key . 'TimeStamp' } = $Self->{TimeObject}->SystemTime2TimeStamp(
+                        SystemTime => $SystemMaintenance->{$Key},
                     );
-                    $DateTimeObject->ToTimeZone( TimeZone => $Self->{UserTimeZone} );
-
-                    $SystemMaintenance->{ $Key . 'TimeStamp' } = $DateTimeObject->ToString();
                 }
 
                 # create blocks
-                $LayoutObject->Block(
+                $Self->{LayoutObject}->Block(
                     Name => 'ViewRow',
                     Data => {
                         %{$SystemMaintenance},
@@ -480,24 +422,12 @@ sub Run {
         }
 
         # generate output
-        my $Output = $LayoutObject->Header();
-        $Output .= $LayoutObject->NavigationBar();
-
-        if ( ( $ParamObject->GetParam( Param => 'Notification' ) || '' ) eq 'Update' ) {
-            $Output .= $LayoutObject->Notify(
-                Info => Translatable('System Maintenance was updated successfully!')
-            );
-        }
-        elsif ( ( $ParamObject->GetParam( Param => 'Notification' ) || '' ) eq 'Add' ) {
-            $Output .= $LayoutObject->Notify(
-                Info => Translatable('System Maintenance was added successfully!')
-            );
-        }
-
-        $Output .= $LayoutObject->Output(
+        my $Output = $Self->{LayoutObject}->Header();
+        $Output .= $Self->{LayoutObject}->NavigationBar();
+        $Output .= $Self->{LayoutObject}->Output(
             TemplateFile => 'AdminSystemMaintenance',
         );
-        $Output .= $LayoutObject->Footer();
+        $Output .= $Self->{LayoutObject}->Footer();
         return $Output;
     }
 
@@ -506,9 +436,6 @@ sub Run {
 sub _ShowEdit {
     my ( $Self, %Param ) = @_;
 
-    my $LayoutObject  = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
-    my $SessionObject = $Kernel::OM->Get('Kernel::System::AuthSession');
-
     # get SystemMaintenance information
     my $SystemMaintenanceData = $Param{SystemMaintenanceData} || {};
 
@@ -532,37 +459,37 @@ sub _ShowEdit {
     }
 
     # start date info
-    $Param{StartDateString} = $LayoutObject->BuildDateSelection(
+    $Param{StartDateString} = $Self->{LayoutObject}->BuildDateSelection(
         %{$SystemMaintenanceData},
         %{ $TimeConfig{StartDate} },
         Prefix           => 'StartDate',
         Format           => 'DateInputFormatLong',
         YearPeriodPast   => 0,
-        YearPeriodFuture => 1,
+        YearPeriodFuture => 0,
         StartDateClass   => $Param{StartDateInvalid} || ' ',
         Validate         => 1,
     );
 
     # stop date info
-    $Param{StopDateString} = $LayoutObject->BuildDateSelection(
+    $Param{StopDateString} = $Self->{LayoutObject}->BuildDateSelection(
         %{$SystemMaintenanceData},
         %{ $TimeConfig{StopDate} },
         Prefix           => 'StopDate',
         Format           => 'DateInputFormatLong',
         YearPeriodPast   => 0,
-        YearPeriodFuture => 1,
+        YearPeriodFuture => 0,
         StopDateClass    => $Param{StopDateInvalid} || ' ',
         Validate         => 1,
     );
 
     # get valid list
-    my %ValidList = $Kernel::OM->Get('Kernel::System::Valid')->ValidList();
+    my %ValidList = $Self->{ValidObject}->ValidList();
 
-    $Param{ValidOption} = $LayoutObject->BuildSelection(
+    $Param{ValidOption} = $Self->{LayoutObject}->BuildSelection(
         Data       => \%ValidList,
         Name       => 'ValidID',
         SelectedID => $SystemMaintenanceData->{ValidID} || 1,
-        Class      => 'Modernize Validate_Required ' . ( $Param{ValidIDServerError} || '' ),
+        Class      => 'Validate_Required ' . ( $Param{ValidIDServerError} || '' ),
     );
 
     if (
@@ -573,20 +500,20 @@ sub _ShowEdit {
         $Param{Checked} = 'checked="checked"';
     }
 
-    my $Output = $LayoutObject->Header();
-    $Output .= $LayoutObject->NavigationBar();
+    my $Output = $Self->{LayoutObject}->Header();
+    $Output .= $Self->{LayoutObject}->NavigationBar();
 
     # show notifications if any
     if ( $Param{NotifyData} ) {
         for my $Notification ( @{ $Param{NotifyData} } ) {
-            $Output .= $LayoutObject->Notify(
+            $Output .= $Self->{LayoutObject}->Notify(
                 %{$Notification},
             );
         }
     }
 
     # get all sessions
-    my @List     = $SessionObject->GetAllSessionIDs();
+    my @List     = $Self->{SessionObject}->GetAllSessionIDs();
     my $Table    = '';
     my $Counter  = @List;
     my %MetaData = ();
@@ -600,23 +527,20 @@ sub _ShowEdit {
 
         for my $SessionID (@List) {
             my $List = '';
-            my %Data = $SessionObject->GetSessionIDData( SessionID => $SessionID );
-            if ( $Data{UserType} && $Data{UserLogin} ) {
-                $MetaData{"$Data{UserType}Session"}++;
-                if ( !$MetaData{"$Data{UserLogin}"} ) {
-                    $MetaData{"$Data{UserType}SessionUniq"}++;
-                    $MetaData{"$Data{UserLogin}"} = 1;
-                }
+            my %Data = $Self->{SessionObject}->GetSessionIDData( SessionID => $SessionID );
+            $MetaData{"$Data{UserType}Session"}++;
+            if ( !$MetaData{"$Data{UserLogin}"} ) {
+                $MetaData{"$Data{UserType}SessionUniq"}++;
+                $MetaData{"$Data{UserLogin}"} = 1;
             }
 
-            $Data{UserType} = 'Agent' if ( !$Data{UserType} || $Data{UserType} ne 'Customer' );
+            $Data{UserType} = 'Agent' if ( $Data{UserType} ne 'Customer' );
 
             # store data to be used later for showing a users session table
             push @UserSessions, {
                 SessionID     => $SessionID,
                 UserFirstname => $Data{UserFirstname},
                 UserLastname  => $Data{UserLastname},
-                UserFullname  => $Data{UserFullname},
                 UserType      => $Data{UserType},
             };
         }
@@ -625,7 +549,7 @@ sub _ShowEdit {
         for my $UserSession (@UserSessions) {
 
             # create blocks
-            $LayoutObject->Block(
+            $Self->{LayoutObject}->Block(
                 Name => $UserSession->{UserType} . 'Session',
                 Data => {
                     %{$UserSession},
@@ -637,7 +561,7 @@ sub _ShowEdit {
         # no customer sessions found
         if ( !$MetaData{CustomerSession} ) {
 
-            $LayoutObject->Block(
+            $Self->{LayoutObject}->Block(
                 Name => 'CustomerNoDataRow',
             );
         }
@@ -645,13 +569,13 @@ sub _ShowEdit {
         # no agent sessions found
         if ( !$MetaData{UserSession} ) {
 
-            $LayoutObject->Block(
+            $Self->{LayoutObject}->Block(
                 Name => 'AgentNoDataRow',
             );
         }
     }
 
-    $Output .= $LayoutObject->Output(
+    $Output .= $Self->{LayoutObject}->Output(
         TemplateFile => "AdminSystemMaintenance$Param{Action}",
         Data         => {
             Counter => $Counter,
@@ -661,7 +585,7 @@ sub _ShowEdit {
         },
     );
 
-    $Output .= $LayoutObject->Footer();
+    $Output .= $Self->{LayoutObject}->Footer();
 
     return $Output;
 }
@@ -679,9 +603,9 @@ sub _GetParams {
         Comment LoginMessage ShowLoginMessage NotifyMessage ValidID )
         )
     {
-        $GetParam->{$ParamName} = $Kernel::OM->Get('Kernel::System::Web::Request')->GetParam( Param => $ParamName );
+        my $EmptyValue = ( $ParamName eq 'ShowLoginMessage' ? 0 : undef );
+        $GetParam->{$ParamName} = $Self->{ParamObject}->GetParam( Param => $ParamName ) || $EmptyValue;
     }
-    $Param{ShowLoginMessage} ||= 0;
 
     ITEM:
     for my $Item (qw(StartDate StopDate)) {
@@ -700,20 +624,22 @@ sub _GetParams {
             $DateStructure{$Period} = $GetParam->{ $Item . $Period };
         }
 
-        my $DateTimeObject = $Kernel::OM->Create(
-            'Kernel::System::DateTime',
-            ObjectParams => {
-                %DateStructure,
-                TimeZone => $Self->{UserTimeZone},
-            },
-        );
-        if ( !$DateTimeObject ) {
+        # check date
+        if ( !$Self->{TimeObject}->Date2SystemTime( %DateStructure, Second => 0 ) ) {
             $Param{Error}->{ $Item . 'Invalid' } = 'ServerError';
             next ITEM;
         }
 
-        $GetParam->{$Item} = $DateTimeObject->ToEpoch();
-        $GetParam->{ $Item . 'TimeStamp' } = $DateTimeObject->ToString();
+        # try to convert date to a SystemTime
+        $GetParam->{$Item} = $Self->{TimeObject}->Date2SystemTime(
+            %DateStructure,
+            Second => 0,
+        );
+
+        # try to convert SystemTime to TimeStamp
+        $GetParam->{ $Item . 'TimeStamp' } = $Self->{TimeObject}->SystemTime2TimeStamp(
+            SystemTime => $GetParam->{$Item},
+        );
     }
 
     return $GetParam;
