@@ -1,6 +1,6 @@
 # --
 # Kernel/Output/HTML/NotificationUIDCheck.pm
-# Copyright (C) 2001-2013 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2015 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -12,9 +12,6 @@ package Kernel::Output::HTML::NotificationUIDCheck;
 use strict;
 use warnings;
 
-use vars qw($VERSION);
-$VERSION = qw($Revision: 1.12 $) [1];
-
 sub new {
     my ( $Type, %Param ) = @_;
 
@@ -38,9 +35,11 @@ sub Run {
     # show error notfy, don't work with user id 1
     return $Self->{LayoutObject}->Notify(
         Priority => 'Error',
-        Link     => '$Env{"Baselink"}Action=AdminUser',
+        Link     => $Self->{LayoutObject}->{Baselink} . 'Action=AdminUser',
         Data =>
-            '$Text{"Don\'t use the Superuser account to work with OTRS! Create new Agents and work with these accounts instead."}',
+            $Self->{LayoutObject}->{LanguageObject}->Translate(
+            "Don't use the Superuser account to work with OTRS! Create new Agents and work with these accounts instead."
+            ),
     );
 }
 
